function [ca,sa,kk,kk1] = cinterp(klo,kup,func,M,ip)

%This program returns the chebyshev polynomial coefficients
%for interpolating a function, func(k), for klo .le. k .le. kup.
%The coefficients are ca=ca(1),...,ca(M), in the following relation:

%  p(k) = sum(i=0 to M-1) ca(i+1)*Ti(psi(k))

%where Ti is the i-th Chebyshev polynomial, and
%psi(k)=2*((k-kup)/(kup-klo))-1;
%k=klo+(kup-klo)*(psi(k)+1)/2

%it also returns the coefficients of the standard
%polynomial interpolation, sa, in

%  p(k) = sum(i=0 to M-1) sa(i+1)*(psi(k)^i)

%kk are the grid points used for the Chebyshev interpolation
%kk1 are the grid points for standard interpolation
%ip is a parameter to be passed on to func.
X=zeros(M,M);
f=zeros(M,1);
f1=zeros(M,1);
for i = 1:M

 x=cos(pi*(i-.5)/M);
 k=klo+(kup-klo)*(x+1)/2;
 kk(i)=k;
 xx(i)=x;
 f(i)=feval(func,k,ip);
 X(i,1)=1;
 X(i,2)=x;
 for j=3:M
  X(i,j)=2*x*X(i,j-1)-X(i,j-2);
 end

end
ca=(X'*X)\X'*f;
if ip == 4; break; end
% now do fixed-interval interpolation:

for i = 1:M

 x=(2*i-M-1)/(M-1);
 k=klo+(kup-klo)*(x+1)/2;
 kk1(i)=k;
 xx1(i)=x;
 f1(i)=feval(func,k,ip);
 X(i,1)=1;
 X(i,2)=x;
 for j=3:M
  X(i,j)=x^(j-1);
 end

end
sa=(X'*X)\X'*f1;

